@@ -42,13 +42,15 @@ class BaseModelColumns(IntEnum):
     HEADING_TEXT = 2
 
 
-class BaseManager(object):
+class BaseManager(GObject.GObject):
 
     def __repr__(self):
         return '<BaseManager>'
 
     @log
     def __init__(self, id_, label, entry):
+        super().__init__()
+
         self._id = id_
         self._label = label
         self.entry = entry
@@ -72,7 +74,7 @@ class BaseManager(object):
             model[iter_][0, 1, 2] = value
         self.selected_id = self.values[1][BaseModelColumns.ID]
 
-    @property
+    @GObject.Property
     @log
     def active(self):
         return self.selected_id
@@ -132,7 +134,7 @@ class SourceManager(BaseManager):
         self._model[iter_][0, 1, 2] = value
         self.values.append(value)
 
-    @property
+    @GObject.Property
     @log
     def active(self):
         return super().active
@@ -277,7 +279,9 @@ class DropDown(Gtk.Revealer):
     def do_select(self, manager, id_):
         manager.active = id_
         if manager == self._source_manager:
-            self._search_filter.view.set_sensitive(id == 'grl-tracker-source')
+            self._search_filter.view.set_sensitive(id_ == 'grl-tracker-source')
+            self.search_manager.active = (
+                'search_all' if id_ != 'grl-tracker-source' else '')
 
 
 class Searchbar(Gtk.SearchBar):
@@ -345,7 +349,7 @@ class Searchbar(Gtk.SearchBar):
         else:
             fields_filter = 'search_all'
 
-        stack = self._stack_switcher.get_stack()
+        stack = self._stack_switcher.props.stack
         if search_term != "":
             stack.set_visible_child_name('search')
             view = stack.get_visible_child()
