@@ -23,7 +23,7 @@
 # delete this exception statement from your version.
 
 from gettext import gettext as _, ngettext
-from gi.repository import Gio, GLib, GObject, Gtk, Pango
+from gi.repository import Gio, GLib, GObject, Gtk, Gdk, Pango
 
 from gnomemusic import log
 from gnomemusic.grilo import grilo
@@ -76,6 +76,8 @@ class PlaylistView(BaseView):
         self._name_label = builder.get_object('playlist_name')
         self._rename_entry = builder.get_object('playlist_rename_entry')
         self._rename_entry.connect('changed', self._on_rename_entry_changed)
+        self._rename_entry.connect(
+            'key-press-event', self._on_rename_entry_key_pressed)
         self._rename_done_button = builder.get_object(
             'playlist_rename_done_button')
         self._songs_count_label = builder.get_object('songs_count')
@@ -145,7 +147,7 @@ class PlaylistView(BaseView):
         self.model.connect('row-inserted', self._on_song_inserted)
         self.model.connect('row-deleted', self._on_song_deleted)
 
-        self.player.connect('playlist-item-changed', self._update_model)
+        self.player.connect('song-changed', self._update_model)
         playlists.connect('playlist-created', self._on_playlist_created)
         playlists.connect('playlist-updated', self._on_playlist_update)
         playlists.connect(
@@ -197,7 +199,7 @@ class PlaylistView(BaseView):
 
         column_star = Gtk.TreeViewColumn()
         self._view.append_column(column_star)
-        self._star_handler.add_star_renderers(self._view, column_star)
+        self._star_handler.add_star_renderers(column_star)
 
         duration_renderer = Gtk.CellRendererText(xpad=32, xalign=1.0)
         column_duration = Gtk.TreeViewColumn()
@@ -250,18 +252,12 @@ class PlaylistView(BaseView):
         if model[_iter][11] == DiscoveryStatus.FAILED:
             cell.set_property('icon-name', self._error_icon_name)
             cell.set_visible(True)
-        elif model[_iter][5].get_url() == self.player.current_track_uri:
+        elif model[_iter][5].get_url() == self.player.url:
             cell.set_property('icon-name', self._now_playing_icon_name)
             cell.set_visible(True)
         else:
             cell.set_visible(False)
 
-    @log
-    def _populate(self):
-        self._init = True
-        self._window.notifications_popup.push_loading()
-        self.populate()
-
     @log
     def _update_model(self, player, playlist, current_iter):
         if self._iter_to_clean:
@@ -415,7 +411,7 @@ class PlaylistView(BaseView):
         # update player's playlist.
         if self.player.running_playlist(
                 'Playlist', self._current_playlist.get_id()):
-            playing_old_path = self.player.currentTrack.get_path().to_string()
+            playing_old_path = self.player.current_song.get_path().to_string()
             playing_old_pos = int(playing_old_path)
             iter_ = model.get_iter_from_string(playing_old_path)
             # if playing song position has changed
@@ -688,8 +684,8 @@ class PlaylistView(BaseView):
         self._sidebar.remove(selection)
 
         if self.player.running_playlist('Playlist', playlist_id):
-            self.player.Stop()
-            self.player.actionbar.set_visible(False)
+            self.player.stop()
+            self.set_player_visible(False)
 
         if row_next:
             self._sidebar.select_row(row_next)
@@ -740,8 +736,8 @@ class PlaylistView(BaseView):
                 song_todelete['playlist'], [song_todelete['song']])
             self._songs_todelete.pop(media_id)
 
+    @GObject.Property(type=bool, default=False)
     @log
-    @property
     def rename_active(self):
         """Indicate if renaming dialog is active"""
         return self._name_stack.get_visible_child_name() == 'renaming_dialog'
@@ -750,6 +746,11 @@ class PlaylistView(BaseView):
     def _on_rename_entry_changed(self, selection):
         self._rename_done_button.set_sensitive(selection.get_text_length() > 0)
 
+    @log
+    def _on_rename_entry_key_pressed(self, widget, event):
+        if event.keyval == Gdk.KEY_Escape:
+            self.disable_rename_playlist()
+
     @log
     def disable_rename_playlist(self):
         """disables rename button and entry"""
@@ -821,4 +822,5 @@ class PlaylistView(BaseView):
         """Populate sidebar.
         Do not reload playlists already displayed.
         """
+        self._window.notifications_popup.push_loading()
         grilo.populate_playlists(self._offset, self._add_playlist_item)
