@@ -27,6 +27,7 @@ from gi.repository import GObject, Gtk
 from gnomemusic import log
 from gnomemusic.albumartcache import Art
 from gnomemusic.grilo import grilo
+from gnomemusic.gstplayer import Playback
 from gnomemusic.widgets.coverstack import CoverStack
 from gnomemusic.widgets.disclistboxwidget import DiscBox
 import gnomemusic.utils as utils
@@ -42,7 +43,7 @@ class ArtistAlbumWidget(Gtk.Box):
         return '<ArtistAlbumWidget>'
 
     @log
-    def __init__(self, media, player, model, header_bar,
+    def __init__(self, media, player, model, parent_view, header_bar,
                  selection_mode_allowed, size_group=None,
                  cover_size_group=None):
         super().__init__(orientation=Gtk.Orientation.HORIZONTAL)
@@ -55,6 +56,7 @@ class ArtistAlbumWidget(Gtk.Box):
         self._artist = utils.get_artist_name(self._media)
         self._album_title = utils.get_album_title(self._media)
         self._model = model
+        self._parent_view = parent_view
         self._header_bar = header_bar
         self._selection_mode = False
         self._selection_mode_allowed = selection_mode_allowed
@@ -77,11 +79,9 @@ class ArtistAlbumWidget(Gtk.Box):
             self._selection_mode_allowed)
 
         ui.get_object('title').set_label(self._album_title)
-        creation_date = self._media.get_creation_date()
-        if creation_date:
-            year = creation_date.get_year()
-            ui.get_object('year').set_markup(
-                '<span color=\'grey\'>{}</span>'.format(year))
+        year = utils.get_media_year(self._media)
+        if year:
+            ui.get_object('year').set_label(year)
 
         if self._size_group:
             self._size_group.add_widget(ui.get_object('box1'))
@@ -97,9 +97,10 @@ class ArtistAlbumWidget(Gtk.Box):
         disc_box = DiscBox(self._model)
         disc_box.set_songs(disc_songs)
         disc_box.set_disc_number(disc_nr)
-        disc_box.set_columns(2)
-        disc_box.show_duration(False)
-        disc_box.show_favorites(False)
+        disc_box.props.columns = 2
+        disc_box.props.show_durations = False
+        disc_box.props.show_favorites = False
+        disc_box.props.show_song_numbers = True
         disc_box.connect('song-activated', self._song_activated)
         disc_box.connect('selection-toggle', self._selection_mode_toggled)
 
@@ -121,17 +122,15 @@ class ArtistAlbumWidget(Gtk.Box):
         """Selection mode button clicked callback."""
         if button.get_active():
             self._selection_mode = True
-            self._disc_listbox.set_selection_mode(True)
-            self._header_bar.set_selection_mode(True)
-            self._player.actionbar.set_visible(False)
-            self._header_bar.header_bar.set_custom_title(
-                self._header_bar._selection_menu_button)
+            self._disc_listbox.props.selection_mode = True
+            self._header_bar.props.selection_mode = True
+            self._parent_view.set_player_visible(False)
         else:
             self._selection_mode = False
-            self._disc_listbox.set_selection_mode(False)
-            self._header_bar.set_selection_mode(False)
-            if(self._player.get_playback_status() != 2):
-                self._player.actionbar.set_visible(True)
+            self._disc_listbox.props.selection_mode = False
+            self._header_bar.props.selection_mode = False
+            if self._player.get_playback_status() != Playback.STOPPED:
+                self._parent_view.set_player_visible(True)
 
     @log
     def _add_item(self, source, prefs, song, remaining, data=None):
@@ -151,15 +150,14 @@ class ArtistAlbumWidget(Gtk.Box):
             disc = self.create_disc_box(disc_nr, discs[disc_nr])
             self._disc_listbox.add(disc)
             if len(discs) == 1:
-                disc.show_disc_label(False)
+                disc.props.show_disc_label = False
 
         if remaining == 0:
             self.emit("songs-loaded")
 
     @log
     def _song_activated(self, widget, song_widget):
-        if (not song_widget.can_be_played
-                or self._selection_mode):
+        if self._selection_mode:
             return
 
         self._player.stop()
@@ -175,4 +173,4 @@ class ArtistAlbumWidget(Gtk.Box):
             return
         self._selection_mode = selection_mode
 
-        self._disc_listbox.set_selection_mode(selection_mode)
+        self._disc_listbox.props.selection_mode = selection_mode
