@@ -22,73 +22,13 @@
 # code, but you are not obligated to do so.  If you do not wish to do so,
 # delete this exception statement from your version.
 
-
 from gettext import gettext as _
 from gi.repository import Gdk, GObject, Gtk
 
 from gnomemusic import log
-from gnomemusic.grilo import grilo
+from gnomemusic.widgets.songwidget import SongWidget
 import gnomemusic.utils as utils
 
-NOW_PLAYING_ICON_NAME = 'media-playback-start-symbolic'
-ERROR_ICON_NAME = 'dialog-error-symbolic'
-
-
-class StarImage(Gtk.Image):
-    """GtkImage for starring songs"""
-    __gtype_name__ = 'StarImage'
-
-    def __repr__(self):
-        return '<StarImage>'
-
-    @log
-    def __init__(self):
-        super().__init__(self)
-
-        self._favorite = False
-
-        self.get_style_context().add_class("star")
-        self.show_all()
-
-    @log
-    def set_favorite(self, favorite):
-        """Set favorite
-
-        Set the current widget as favorite or not.
-
-        :param bool favorite: Value to switch the widget state to
-        """
-        self._favorite = favorite
-
-        if self._favorite:
-            self.set_state_flags(Gtk.StateFlags.SELECTED, False)
-        else:
-            self.unset_state_flags(Gtk.StateFlags.SELECTED)
-
-    @log
-    def get_favorite(self):
-        """Return the current state of the widget
-
-        :return: The state of the widget
-        :rtype: bool
-        """
-        return self._favorite
-
-    @log
-    def toggle_favorite(self):
-        """Toggle the widget state"""
-        self._favorite = not self._favorite
-
-        self.set_favorite(self._favorite)
-
-    @log
-    def hover(self, widget, event, data):
-        self.set_state_flags(Gtk.StateFlags.PRELIGHT, False)
-
-    @log
-    def unhover(self, widget, event, data):
-        self.unset_state_flags(Gtk.StateFlags.PRELIGHT)
-
 
 class DiscSongsFlowBox(Gtk.FlowBox):
     """FlowBox containing the songs on one disc
@@ -107,14 +47,25 @@ class DiscSongsFlowBox(Gtk.FlowBox):
 
         :param int columns: The number of columns the widget uses
         """
-        super().__init__(self)
+        super().__init__()
         super().set_selection_mode(Gtk.SelectionMode.NONE)
 
         self._columns = columns
         self.get_style_context().add_class('discsongsflowbox')
 
+    @GObject.Property(type=int, minimum=1, default=1)
     @log
-    def set_columns(self, columns):
+    def columns(self):
+        """Number of columns for the song list
+
+        :returns: The number of columns
+        :rtype: int
+        """
+        return self._columns
+
+    @columns.setter
+    @log
+    def columns(self, columns):
         """Set the number of columns to use
 
         :param int columns: The number of columns the widget uses
@@ -143,10 +94,16 @@ class DiscBox(Gtk.Box):
     __gsignals__ = {
         'selection-changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'selection-toggle': (GObject.SignalFlags.RUN_FIRST, None, ()),
-        'track-activated': (GObject.SignalFlags.RUN_FIRST, None, (Gtk.Widget,))
-
+        'song-activated': (GObject.SignalFlags.RUN_FIRST, None, (Gtk.Widget,))
     }
 
+    columns = GObject.Property(type=int, minimum=1, default=1)
+    selection_mode = GObject.Property(type=bool, default=False)
+    show_disc_label = GObject.Property(type=bool, default=False)
+    show_durations = GObject.Property(type=bool, default=False)
+    show_favorites = GObject.Property(type=bool, default=False)
+    show_song_numbers = GObject.Property(type=bool, default=False)
+
     def __repr__(self):
         return '<DiscBox>'
 
@@ -156,7 +113,7 @@ class DiscBox(Gtk.Box):
 
         :param model: The TreeStore to use
         """
-        super().__init__(self)
+        super().__init__()
 
         self._model = model
         self._model.connect('row-changed', self._model_row_changed)
@@ -168,94 +125,39 @@ class DiscBox(Gtk.Box):
         self._label.set_no_show_all(True)
         self._disc_songs_flowbox = builder.get_object('discsongsflowbox')
 
-        self._selection_mode = False
+        self.bind_property(
+            'columns', self._disc_songs_flowbox, 'columns',
+            GObject.BindingFlags.SYNC_CREATE)
+        self.bind_property(
+            'show-disc-label', self._label, 'visible',
+            GObject.BindingFlags.SYNC_CREATE)
+
         self._selection_mode_allowed = True
         self._selected_items = []
         self._songs = []
 
         self.pack_start(builder.get_object('disc'), True, True, 0)
 
-    @log
-    def set_columns(self, columns):
-        """Set the number of columns used by the songs list
-
-        :param int columns: Number of columns to display
-        """
-        self._disc_songs_flowbox.set_columns(columns)
-
     @log
     def set_disc_number(self, disc_number):
         """Set the dics number to display
 
         :param int disc_number: Disc number to display
         """
-        self._label.set_markup(_("Disc {}").format(disc_number))
-        self._label.get_style_context().add_class('dim-label')
+        self._label.set_label(_("Disc {}").format(disc_number))
         self._label.set_visible(True)
 
     @log
-    def show_disc_label(self, show_header):
-        """Wheter to show the disc number label
-
-        :param bool show_header: Display the disc number label
-        """
-        self._label.set_visible(False)
-        self._label.hide()
-
-    @log
-    def show_duration(self, show_duration):
-        """Wheter to show the song durations
-
-        :param bool show_duration: Display the song durations
-        """
-        def child_show_duration(child):
-            child.get_child().duration.set_visible(show_duration)
-
-        self._disc_songs_flowbox.foreach(child_show_duration)
-
-    @log
-    def show_favorites(self, show_favorites):
-        """Where to show the favorite switches
-
-        :param bool show_favorites: Display the favorite
-        switches
-        """
-        def child_show_favorites(child):
-            child.get_child().starevent.set_visible(show_favorites)
-
-        self._disc_songs_flowbox.foreach(child_show_favorites)
-
-    @log
-    def show_song_numbers(self, show_song_number):
-        """Whether to show the song numbers
-
-        :param bool show_song_number: Display the song number
-        """
-        def child_show_song_number(child):
-            child.get_child().number.set_visible(show_song_number)
-
-        self._disc_songs_flowbox.foreach(child_show_song_number)
-
-    @log
-    def set_tracks(self, tracks):
+    def set_songs(self, songs):
         """Songs to display
 
-        :param list tracks: A list of Grilo media items to
+        :param list songs: A list of Grilo media items to
         add to the widget
         """
-        for track in tracks:
-            song_widget = self._create_song_widget(track)
+        for song in songs:
+            song_widget = self._create_song_widget(song)
             self._disc_songs_flowbox.insert(song_widget, -1)
-            track.song_widget = song_widget
-
-    @log
-    def set_selection_mode(self, selection_mode):
-        """Set selection mode
-
-        :param bool selection_mode: Allow selection mode
-        """
-        self._selection_mode = selection_mode
-        self._disc_songs_flowbox.foreach(self._toggle_widget_selection)
+            song.song_widget = song_widget
 
     @log
     def get_selected_items(self):
@@ -273,7 +175,7 @@ class DiscBox(Gtk.Box):
     def _get_selected(self, child):
         song_widget = child.get_child()
 
-        if song_widget.check_button.get_active():
+        if song_widget.selected:
             itr = song_widget.itr
             self._selected_items.append(self._model[itr][5])
 
@@ -298,85 +200,45 @@ class DiscBox(Gtk.Box):
         self._disc_songs_flowbox.foreach(child_select_none)
 
     @log
-    def _create_song_widget(self, track):
+    def _create_song_widget(self, song):
         """Helper function to create a song widget for a
         single song
 
-        :param track: A Grilo media item
+        :param song: A Grilo media item
         :returns: A complete song widget
         :rtype: Gtk.EventBox
         """
-        builder = Gtk.Builder()
-        builder.add_from_resource('/org/gnome/Music/TrackWidget.ui')
-        song_widget = builder.get_object('eventbox1')
+        song_widget = SongWidget(song)
         self._songs.append(song_widget)
 
-        title = utils.get_media_title(track)
+        title = utils.get_media_title(song)
 
         itr = self._model.append(None)
 
-        self._model[itr][0, 1, 2, 5, 6] = [title, '', '', track, False]
+        self._model[itr][0, 1, 2, 5, 6] = [title, '', '', song, False]
 
         song_widget.itr = itr
         song_widget.model = self._model
+        song_widget.connect('button-release-event', self._song_activated)
+        song_widget.connect('selection-changed', self._on_selection_changed)
 
-        track_number = track.get_track_number()
-        if track_number == 0:
-            track_number = ""
-        song_widget.number = builder.get_object('num')
-        song_widget.number.set_markup(
-            '<span color=\'grey\'>{}</span>'.format(track_number))
-        song_widget.number.set_no_show_all(True)
+        self.bind_property(
+            'selection-mode', song_widget, 'selection-mode',
+            GObject.BindingFlags.SYNC_CREATE)
+        self.bind_property(
+            'show-durations', song_widget, 'show-duration',
+            GObject.BindingFlags.SYNC_CREATE)
+        self.bind_property(
+            'show-favorites', song_widget, 'show-favorite',
+            GObject.BindingFlags.SYNC_CREATE)
+        self.bind_property(
+            'show-song-numbers', song_widget, 'show-song-number',
+            GObject.BindingFlags.SYNC_CREATE)
 
-        song_widget.title = builder.get_object('title')
-        song_widget.title.set_text(title)
-        song_widget.title.set_max_width_chars(50)
-
-        song_widget.duration = builder.get_object('duration')
-        time = utils.seconds_to_string(track.get_duration())
-        song_widget.duration.set_text(time)
-
-        song_widget.check_button = builder.get_object('select')
-        song_widget.check_button.set_visible(False)
-        song_widget.check_button.connect('toggled',
-                                         self._check_button_toggled,
-                                         song_widget)
-
-        song_widget.now_playing_sign = builder.get_object('image1')
-        song_widget.now_playing_sign.set_from_icon_name(
-            NOW_PLAYING_ICON_NAME,
-            Gtk.IconSize.SMALL_TOOLBAR)
-        song_widget.now_playing_sign.set_no_show_all(True)
-        song_widget.can_be_played = True
-        song_widget.connect('button-release-event', self._track_activated)
-
-        song_widget.star_image = builder.get_object('starimage')
-        song_widget.star_image.set_favorite(track.get_favourite())
-        song_widget.star_image.set_visible(True)
-
-        song_widget.starevent = builder.get_object('starevent')
-        song_widget.starevent.connect('button-release-event',
-                                      self._toggle_favorite,
-                                      song_widget)
-        song_widget.starevent.connect('enter-notify-event',
-                                      song_widget.star_image.hover, None)
-        song_widget.starevent.connect('leave-notify-event',
-                                      song_widget.star_image.unhover, None)
         return song_widget
 
     @log
-    def _toggle_favorite(self, widget, event, song_widget):
-        if event.button == Gdk.BUTTON_PRIMARY:
-            song_widget.star_image.toggle_favorite()
-
-        # FIXME: ugleh. Should probably be triggered by a
-        # signal.
-        favorite = song_widget.star_image.get_favorite()
-        grilo.set_favorite(self._model[song_widget.itr][5], favorite)
-        return True
-
-    @log
-    def _check_button_toggled(self, widget, song_widget):
+    def _on_selection_changed(self, widget):
         self.emit('selection-changed')
 
         return True
@@ -384,25 +246,22 @@ class DiscBox(Gtk.Box):
     @log
     def _toggle_widget_selection(self, child):
         song_widget = child.get_child()
-        song_widget.check_button.set_visible(self._selection_mode)
-        if self._selection_mode == False:
-            if song_widget.check_button.get_active():
-                song_widget.check_button.set_active(False)
+        song_widget.props.selection_mode = self.props.selection_mode
 
     @log
-    def _track_activated(self, widget, event):
+    def _song_activated(self, widget, event):
         # FIXME: don't think keys work correctly, if they did ever
         # even.
         if (not event.button == Gdk.BUTTON_SECONDARY
                 or (event.button == Gdk.BUTTON_PRIMARY
                     and event.state & Gdk.ModifierType.CONTROL_MASK)):
-            self.emit('track-activated', widget)
-            if self._selection_mode:
+            self.emit('song-activated', widget)
+            if self.props.selection_mode:
                 itr = widget.itr
                 self._model[itr][6] = not self._model[itr][6]
         else:
             self.emit('selection-toggle')
-            if self._selection_mode:
+            if self.props.selection_mode:
                 itr = widget.itr
                 self._model[itr][6] = True
 
@@ -410,14 +269,14 @@ class DiscBox(Gtk.Box):
 
     @log
     def _model_row_changed(self, model, path, itr):
-        if (not self._selection_mode
+        if (not self.props.selection_mode
                 or not model[itr][5]):
             return
 
         song_widget = model[itr][5].song_widget
         selected = model[itr][6]
-        if selected != song_widget.check_button.get_active():
-            song_widget.check_button.set_active(selected)
+        if selected != song_widget.props.selected:
+            song_widget.props.selected = selected
 
         return True
 
@@ -450,6 +309,10 @@ class DiscListBox(Gtk.Box):
         super().add(widget)
         widget.connect('selection-changed', self._on_selection_changed)
 
+        self.bind_property(
+            'selection-mode', widget, 'selection-mode',
+            GObject.BindingFlags.SYNC_CREATE)
+
     @log
     def _on_selection_changed(self, widget):
         self.emit('selection-changed')
@@ -486,21 +349,27 @@ class DiscListBox(Gtk.Box):
 
         self.foreach(child_select_none)
 
+    @GObject.Property(type=bool, default=False)
     @log
-    def set_selection_mode(self, selection_mode):
-        """Set selection mode
+    def selection_mode(self):
+        """selection mode getter
 
-        :param bool selection_mode: Allow selection mode
+        :returns: If selection mode is active
+        :rtype: bool
+        """
+        return self._selection_mode
+
+    @selection_mode.setter
+    @log
+    def selection_mode(self, value):
+        """selection-mode setter
+
+        :param bool value: Activate selection mode
         """
         if not self._selection_mode_allowed:
             return
 
-        self._selection_mode = selection_mode
-
-        def set_child_selection_mode(child):
-            child.set_selection_mode(self._selection_mode)
-
-        self.foreach(set_child_selection_mode)
+        self._selection_mode = value
 
     @log
     def set_selection_mode_allowed(self, allowed):
